@@ -23,7 +23,7 @@ use Moo;
 use MooX::Types::MooseLike::Base qw(:all);
 use namespace::clean;
 
-our $VERSION = '0.16';
+our $VERSION = '0.18';
 
 use constant {
     AMW_ORDER_WILDCARD_ERROR => 999999,
@@ -674,6 +674,33 @@ calls C<resume>, which is in charge for the actual uploading.
 
 Restore the state and resume where it was left.
 
+This method accepts an optional list of parameters. Each parameter may be:
+
+=over 4
+
+=item a scalar
+
+This is considered a job id.
+
+=item a hashref
+
+This will be merged in the query to retrieve the pending jobs. A
+sample usage could be:
+
+  $upload->resume({ task => [qw/upload product_deletion/] });
+
+to resume only those specific tasks.
+
+=back
+
+=head2 get_pending_jobs
+
+Return the list of hashref with the pending jobs out of the database.
+Accepts the same parameters as C<resume> (which actually calls this
+method).
+
+=cut
+
 =head1 INTERNAL METHODS
 
 =head2 prepare_feeds($type, { name => $feed_name, content => "<xml>..."}, { name => $feed_name2, content => "<xml>..."}, ....)
@@ -746,15 +773,18 @@ sub upload {
                               variants
                              /) {
         my $method = $feed_name . "_feed";
-        my $content = $feeder->$method;
-        push @feeds, {
-                      name => $feed_name,
-                      content => $content,
-                     }
+        if (my $content = $feeder->$method) {
+            push @feeds, {
+                          name => $feed_name,
+                          content => $content,
+                         };
+        }
     }
-    my $job_id = $self->prepare_feeds(upload => \@feeds);
-    $self->_mark_products_as_pending($job_id, @products);
-    return $job_id;
+    if (my $job_id = $self->prepare_feeds(upload => \@feeds)) {
+        $self->_mark_products_as_pending($job_id, @products);
+        return $job_id;
+    }
+    return;
 }
 
 sub _mark_products_as_pending {
@@ -789,6 +819,7 @@ sub _mark_products_as_pending {
 sub prepare_feeds {
     my ($self, $task, $feeds) = @_;
     die "Missing task ($task) and feeds ($feeds)" unless $task && $feeds;
+    return unless @$feeds; # nothing to do
     my $job_id = $task . "-" . DateTime->now->strftime('%F-%H-%M-%S');
     my $job_started_epoch = time();
 
@@ -834,20 +865,71 @@ sub prepare_feeds {
 }
 
 
-sub resume {
-    my ($self, @jobs) = @_;
+sub get_pending_jobs {
+    my ($self, @args) = @_;
     my %additional;
-    if (@jobs) {
-        $additional{amws_job_id} = { -in => \@jobs };
+    my @named_jobs;
+    foreach my $arg (@args) {
+        if (!ref($arg)) {
+            push @named_jobs, $arg;
+        }
+        elsif (ref($arg) eq 'HASH') {
+            # add the filters
+            foreach my $key (keys %$arg) {
+                if ($additional{$key}) {
+                    die "Attempt to overwrite $key in the additional parameters!\n";
+                }
+                else {
+                    $additional{$key} = $arg->{$key};
+                }
+            }
+        }
+        else {
+            die "Argument must be either a scalar with a job name and/or "
+              . "an hashref with additional filters!";
+        }
     }
-    my ($stmt, @bind) = $self->sqla->select(amazon_mws_jobs => '*', {
-                                                                     aborted => 0,
-                                                                     success => 0,
-                                                                     shop_id => $self->_unique_shop_id,
-                                                                     %additional,
-                                                                    });
+    if (@named_jobs) {
+        $additional{amws_job_id} = { -in => \@named_jobs };
+    }
+    my ($stmt, @bind) = $self->sqla->select(amazon_mws_jobs => '*',
+                                            {
+                                             %additional,
+                                             aborted => 0,
+                                             success => 0,
+                                             shop_id => $self->_unique_shop_id,
+                                            },
+                                            { -asc => 'job_started_epoch'});
     my $pending = $self->_exe_query($stmt, @bind);
+    my %jobs;
     while (my $row = $pending->fetchrow_hashref) {
+        $jobs{$row->{task}} ||= [];
+        push @{$jobs{$row->{task}}}, $row;
+    }
+    my @out;
+    foreach my $task (qw/product_deletion upload shipping_confirmation order_ack/) {
+        if (my $list = delete $jobs{$task}) {
+            if ($task eq 'order_ack') {
+                for (1..2) {
+                    push @out, pop @$list if @$list;
+                }
+            }
+            elsif ($task eq 'shipping_confirmation') {
+                while (@$list) {
+                    push @out, pop @$list;
+                }
+            }
+            else {
+                push @out, @$list if @$list;
+            }
+        }
+    }
+    return @out;
+}
+
+sub resume {
+    my ($self, @args) = @_;
+    foreach my $row ($self->get_pending_jobs(@args)) {
         print "Working on $row->{amws_job_id}\n";
         # check if the job dir exists
         if (-d $self->_feed_job_dir($row->{amws_job_id})) {
@@ -2057,8 +2139,10 @@ Return a list of hashref with two keys, C<amazon_order_id> and
 C<shop_order_id> for each order which is waiting confirmation.
 
 This is implemented looking into amazon_mws_orders where there is no
-shipping confirmation job id but there is the confirmed flag (which
-means we acknowledged the order).
+shipping confirmation job id.
+
+The confirmed flag (which means we acknowledged the order) is ignored
+to avoid stuck order_ack jobs to prevent the shipping confirmation.
 
 =cut
 
@@ -2070,7 +2154,8 @@ sub orders_waiting_for_shipping {
                                                     {
                                                      shop_id => $self->_unique_shop_id,
                                                      shipping_confirmation_job_id => undef,
-                                                     confirmed => 1,
+                                                     # do not stop the unconfirmed to be considered
+                                                     # confirmed => 1,
                                                     }));
     my @out;
     while (my $row = $sth->fetchrow_hashref) {
